from urlparse import urljoin
from django import template
from django.conf import settings

from django_antichaos.utils import get_tagged_models

register = template.Library()

def tagged_models():
    models = [dict(name = m.__name__, model = m, ctype_id = ctype_id)
                   for ctype_id, m in get_tagged_models()]
    return dict(models = models)
tagged_models = register.inclusion_tag('antichaos/tag_cloud_list.html')(tagged_models)

def antichaos_media_prefix():
    return urljoin(settings.MEDIA_URL, 'antichaos/')
antichaos_media_prefix = register.simple_tag(antichaos_media_prefix)




from django.test import TestCase
from django.db import models

from tagging.fields import TagField
from tagging.models import Tag, TaggedItem
from django_antichaos.utils import *


class Post(models.Model):
    title = models.CharField(max_length = 20)
    tags = TagField()

    def __unicode__(self):
        return self.title


class Link(models.Model):
    url = models.CharField(max_length = 20)
    tags = TagField()

    def __unicode__(self):
        return self.url

class TagCloudTests(TestCase):
    def testCreatePost(self):
        self.assertEqual(0, Post.objects.count())
        Post(tags = 'one, two, three').save()
        self.assertEqual(1, Post.objects.count())

    def testContentTypes(self):
        Post(tags = 'one, two, three').save()
        Link(tags = 'six, seven').save()

        mods = [m for ctype_id, m in get_tagged_models()]
        self.assertEqual(2, len(mods))
        self.assert_(Post in mods)
        self.assert_(Link in mods)

class CommandsTests(TestCase):
    def setUp(self):
        self.post_ctype = model_to_ctype(Post)
        self.link_ctype = model_to_ctype(Link)

        self.post1 = Post(title = 'First post', tags = 'one, two, three')
        self.post1.save()
        Post(title = 'Second post', tags = 'five, six').save()

        Link(url = 'http://blah.com', tags = 'five, seven').save()
        Link(url = 'http://minor.com', tags = 'two, three').save()

        self.tagids = dict((tag.name, tag.id) for tag in Tag.objects.all())

    def testMerge(self):
        self.assertEqual(1, TaggedItem.objects.get_by_model(Post, 'two').count())
        self.assertEqual(1, TaggedItem.objects.get_by_model(Post, 'five').count())
        self.assertEqual(1, TaggedItem.objects.get_by_model(Link, 'three').count())
        self.assertEqual(1, TaggedItem.objects.get_by_model(Link, 'five').count())
        tags_after = Post.objects.get(id=self.post1.id).tags
        self.assert_(tags_after == 'one three two' or tags_after == 'one, two, three' )

        t = self.tagids
        process_commands(self.post_ctype, [
            'merge|%s|%s' % (t['three'], t['five']),
            'merge|%s|%s' % (t['two'], t['three']),
        ])

        self.assertEqual(2, TaggedItem.objects.get_by_model(Post, 'two').count())
        self.assertEqual(0, TaggedItem.objects.get_by_model(Post, 'five').count())
        self.assertEqual(1, TaggedItem.objects.get_by_model(Link, 'three').count())
        self.assertEqual(1, TaggedItem.objects.get_by_model(Link, 'five').count())
        self.assertEqual('one two', Post.objects.get(id=self.post1.id).tags)

    def testRename(self):
        self.assertEqual(1, TaggedItem.objects.get_by_model(Post, 'five').count())
        self.assertEqual(1, TaggedItem.objects.get_by_model(Link, 'five').count())
        self.assertEqual(0, TaggedItem.objects.get_by_model(Post, 'new-tag').count())

        t = self.tagids
        process_commands(self.post_ctype, [
            'rename|%s|%s' % (t['five'], 'new-tag'),
        ])

        self.assertEqual(0, TaggedItem.objects.get_by_model(Post, 'five').count())
        self.assertEqual(1, TaggedItem.objects.get_by_model(Link, 'five').count())
        self.assertEqual(1, TaggedItem.objects.get_by_model(Post, 'new-tag').count())




import logging

from django.contrib.contenttypes.models import ContentType
from tagging.models import Tag, TaggedItem
from tagging.fields import TagField
from tagging.utils import edit_string_for_tags
from tagging import settings

def get_tagged_models():
    cids = TaggedItem._default_manager.values('content_type').distinct()
    ctypes = ContentType._default_manager.filter(id__in = cids)
    return [(ctype.id, ctype.model_class()) for ctype in ctypes]


def model_to_ctype(model):
    return ContentType._default_manager.get_for_model(model)


def update_objects_tags(object):
    if object is None:
        return

    object_tags = Tag.objects.get_for_object(object)
    tags_as_string = edit_string_for_tags(object_tags)

    for field in object._meta.fields:
        if isinstance(field, TagField):
            setattr(object, field.attname, tags_as_string)
            object.save()
            break


def process_merge(ctype, to_tag, from_tag):
    logger = logging.getLogger('antichaos.utils')

    to_tag = Tag.objects.get(id=to_tag)
    from_tag = Tag.objects.get(id=from_tag)

    logger.debug('merging tag "%s" to_tag tag "%s"' % (from_tag.name, to_tag.name))

    delete_params = {}
    if hasattr(settings, 'MULTILINGUAL_TAGS') is True:
        """Specialization for tagging-ng's delete method."""
        delete_params['update'] = False

    for item in from_tag.items.filter(content_type = ctype):
        if to_tag.items.filter(
                content_type = ctype,
                object_id = item.object_id).count() != 0:
            logger.debug('item "%s" already binded to_tag tag "%s"' % (item, to_tag))
            item.delete(**delete_params)
        else:
            item.tag = to_tag
            item.save()
            logger.debug('item "%s" merged' % item)

        update_objects_tags(item.object)

def process_rename(ctype, tag_id, new_value):
    logger = logging.getLogger('antichaos.utils')

    new_tag, created = Tag.objects.get_or_create(name = new_value)
    if created:
        logger.debug('tag "%s" was created' % new_value)

    process_merge(ctype, new_tag.id, tag_id)


def process_commands(ctype, commands):
    logger = logging.getLogger('antichaos.utils')
    logger.debug('processing commands')

    processors = dict((name.split('_', 1)[1], value)
        for name, value in globals().iteritems()
            if name.startswith('process_') and name != 'process_commands')

    for cmd in commands:
        kwargs = dict(
            (str(key), value) for key, value in (
                c.split('=') for c in cmd.split(',') if c))
        action = kwargs.pop('action')
        proc = processors.get(action, None)
        if proc is not None:
            proc(ctype, **kwargs)




from pdb import set_trace

from django.http import HttpResponse
from django.shortcuts import render_to_response, redirect
from django.template import RequestContext, TemplateDoesNotExist
from django.contrib.contenttypes.models import ContentType
from django.utils import simplejson
from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse

from django_antichaos.utils import process_commands
from tagging.models import Tag, TaggedItem

def template_list(model, template_name):
    app_label = model._meta.app_label
    module_name = model._meta.module_name
    _locals = locals()
    return [
        'antichaos/%(app_label)s/%(module_name)s/%(template_name)s.html' % _locals,
        'antichaos/%(app_label)s/%(template_name)s.html' % _locals,
        'antichaos/%(template_name)s.html' % _locals,
    ]

def cloud(request, ctype_id):
    admin_index = reverse('admin_index')

    if request.user.is_staff == False:
        return redirect(admin_index)

    ctype = ContentType._default_manager.get(id = ctype_id)

    if request.method == 'POST':
        changes = request.POST.getlist('changes')
        process_commands(ctype, changes)

    json = request.GET.get('json', False)

    model = ctype.model_class()
    objects = Tag.objects.usage_for_model(model, counts = True)

    data = dict(
        title = _('Tag cloud for %s') % _(ctype.model),
        ctype = ctype,
        objects = objects,
        root_path = admin_index,
    )

    if json:
        data['objects'] = [
            dict(
                id = tag.id,
                name = tag.name or tag.name_any,
                count = tag.count,
            ) for tag in objects]
        data['ctype'] = ctype.id
        data = simplejson.dumps(data)
        return HttpResponse(data, 'text/html')

    return render_to_response(
        template_list(model, 'tag_cloud'),
        data,
        context_instance = RequestContext(request))


def preview(request, ctype_id, tag_id):
    ctype = ContentType._default_manager.get(id = ctype_id)

    tag = Tag.objects.get(id = tag_id)

    skip = int(request.GET.get('skip', 0))
    top = int(request.GET.get('top', 5))
    next_skip = skip + top

    model = ctype.model_class()
    objects = TaggedItem.objects.get_by_model(model, tag)

    app_label = model._meta.app_label
    module_name = model._meta.module_name

    return render_to_response(
        template_list(model, 'tag_preview'),
        dict(
            tag = tag,
            ctype = ctype,
            objects = objects[skip:next_skip],
            skip = next_skip,
            top = top,
            more = len(objects[next_skip:]),
        ), context_instance = RequestContext(request))




import logging
import os.path

from django.conf import settings
from django.conf.urls.defaults import *

urlpatterns = patterns('django_antichaos.views',
    (r'^cloud/(?P<ctype_id>\d+)/$', 'cloud', {}, 'tag-cloud'),
    (r'^cloud/(?P<ctype_id>\d+)/preview/(?P<tag_id>\d+)/$', 'preview', {}, 'tag-preview'),
)

# create link in media directory
if not os.path.exists(settings.MEDIA_ROOT):
    os.makedirs(settings.MEDIA_ROOT)

APP_MEDIA_ROOT = os.path.join(
    settings.MEDIA_ROOT,
    'antichaos')

_local = os.path.join(
            os.path.dirname(__file__),
            'media')

_logger = logging.getLogger('antichaos')

if os.path.islink(APP_MEDIA_ROOT):
    _real_path =os.path.realpath(APP_MEDIA_ROOT)
    if _real_path != _local:
        _logger.warning('removing old link %s' % _real_path)
        os.unlink(APP_MEDIA_ROOT)

if not os.path.exists(APP_MEDIA_ROOT):
    _logger.info('linking %r to %r' % (_local, APP_MEDIA_ROOT))
    os.symlink(_local, APP_MEDIA_ROOT)
